import time

import es
import gamethread

info = es.AddonInfo()
info.basename = "spree"
info.name = "L4D Killing Spree counter"
info.author = "GODJonez"

players = {}
polling = False
cooldown = es.ServerVar('spree_cooldown', 5, 'Cooldown in seconds to count for killing spree')
announce_best = es.ServerVar('spree_announce_best', 1, 'Announce the best spree and jackpot at the end of round.')
show_ai = es.ServerVar('spree_non_player', 'AI Director', 'Name to be used for non-player kills. Set to 0 to disable.')
ads = set()
the_best = {
    'spree': (0, None),
    'jackpot': (0, None),
}

def player_death(ev):
    global polling
    userid = int(ev['attacker'])
    if userid not in players:
        players[userid] = [1, time.time(), 0]
    else:
        kills = players[userid][0] + 1
        players[userid][0] = kills
        players[userid][1] = time.time()
    if not ads:
        gamethread.delayed(0, msg_em)
    ads.add(userid)
    if not polling:
        gamethread.delayed(float(cooldown), poll)
        polling = True

def getplayername(userid):
    if userid:
        return es.getplayername(userid)
    else:
        ai_name = str(show_ai)
        if ai_name == '0':
            ai_name = ''
        return ai_name

def msg_em():
    for userid in ads:
        kills, kt, last = players[userid]
        if kills > 1:
            delta = kills-last
            es.tell(userid, 'Killing spree: %s (%+d)'%(kills, delta))
            if delta > 5:
                name = getplayername(userid)
                if name:
                    es.msg('\x04%s\x03 hit a jackpot of %s!'%(getplayername(userid),delta))
                    if delta > the_best['jackpot'][0]:
                        the_best['jackpot'] = delta, name
        players[userid][2] = kills
    ads.clear()

def poll():
    global polling
    now = time.time()
    cd = float(cooldown)
    removes = set()
    mintime = now
    for userid, data in players.iteritems():
        dt = data[1]
        if now-dt>cd:
            kills = data[0]
            if kills > 1:
                name = getplayername(userid)
                if name:
                    es.msg('\x04%s\x03 had a killing spree of %s kills.'%(name, kills))
                    if kills > the_best['spree'][0]:
                        the_best['spree'] = kills, name
            removes.add(userid)
        elif dt < mintime:
            mintime = dt
    for userid in removes:
        del players[userid]
    if not players:
        polling = False
    else:
        gamethread.delayed(cd-now+mintime, poll)

def round_start(ev):
    clear_the_best()

def clear_the_best():
    the_best['spree'] = 0, None
    the_best['jackpot'] = 0, None

def round_end(ev):
    do_the_best()

def map_transition(ev):
    do_the_best()

def do_the_best():
    if int(announce_best):
        if the_best['spree'][0] == 0:
            es.msg('\x01It seems like no one had any killing sprees this time around.')
        else:
            es.msg('\x03Best killing spree: %s kills by \x04%s'%the_best['spree'])
        if the_best['jackpot'][0] == 0:
            es.msg('\x01It seems like no one had any jackpots this time around.')
        else:
            es.msg('\x03Best jackpot: %s kills by \x04%s'%the_best['jackpot'])
    clear_the_best()
